<?php

namespace Superrobota\Tableview\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Superrobota\Tableview\Http\Requests\TableviewRequest;
use Superrobota\Tableview\Interfaces\TableviewRepositoryInterface;
use Superrobota\Tableview\Models\Tableview;

/**
 * Resource controller class for tableview.
 */
class TableviewResourceController extends BaseController
{

    /**
     * Initialize tableview resource controller.
     *
     * @param type TableviewRepositoryInterface $tableview
     *
     * @return null
     */
    public function __construct(TableviewRepositoryInterface $tableview)
    {
        parent::__construct();
        $this->repository = $tableview;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Superrobota\Tableview\Repositories\Criteria\TableviewResourceCriteria::class);
    }

    /**
     * Display a list of tableview.
     *
     * @return Response
     */
    public function index(TableviewRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Superrobota\Tableview\Repositories\Presenter\TableviewPresenter::class)
                ->$function();
        }

        $tableviews = $this->repository->paginate();

        return $this->response->title(trans('tableview::tableview.names'))
            ->view('tableview::tableview.index', true)
            ->data(compact('tableviews'))
            ->output();
    }

    /**
     * Display tableview.
     *
     * @param Request $request
     * @param Model   $tableview
     *
     * @return Response
     */
    public function show(TableviewRequest $request, Tableview $tableview)
    {

        if ($tableview->exists) {
            $view = 'tableview::tableview.show';
        } else {
            $view = 'tableview::tableview.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('tableview::tableview.name'))
            ->data(compact('tableview'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new tableview.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(TableviewRequest $request)
    {

        $tableview = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('tableview::tableview.name')) 
            ->view('tableview::tableview.create', true) 
            ->data(compact('tableview'))
            ->output();
    }

    /**
     * Create new tableview.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(TableviewRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $tableview                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('tableview::tableview.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('tableview/tableview/' . $tableview->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/tableview/tableview'))
                ->redirect();
        }

    }

    /**
     * Show tableview for editing.
     *
     * @param Request $request
     * @param Model   $tableview
     *
     * @return Response
     */
    public function edit(TableviewRequest $request, Tableview $tableview)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('tableview::tableview.name'))
            ->view('tableview::tableview.edit', true)
            ->data(compact('tableview'))
            ->output();
    }

    /**
     * Update the tableview.
     *
     * @param Request $request
     * @param Model   $tableview
     *
     * @return Response
     */
    public function update(TableviewRequest $request, Tableview $tableview)
    {
        try {
            $attributes = $request->all();

            $tableview->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('tableview::tableview.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('tableview/tableview/' . $tableview->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('tableview/tableview/' . $tableview->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the tableview.
     *
     * @param Model   $tableview
     *
     * @return Response
     */
    public function destroy(TableviewRequest $request, Tableview $tableview)
    {
        try {

            $tableview->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('tableview::tableview.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('tableview/tableview/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('tableview/tableview/' . $tableview->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple tableview.
     *
     * @param Model   $tableview
     *
     * @return Response
     */
    public function delete(TableviewRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('tableview::tableview.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('tableview/tableview'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/tableview/tableview'))
                ->redirect();
        }

    }

    /**
     * Restore deleted tableviews.
     *
     * @param Model   $tableview
     *
     * @return Response
     */
    public function restore(TableviewRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('tableview::tableview.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/tableview/tableview'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/tableview/tableview/'))
                ->redirect();
        }

    }

}
